/**
 * JHOVE2 - Next-generation architecture for format-aware characterization
 *
 * Copyright (c) 2009 by The Regents of the University of California,
 * Ithaka Harbors, Inc., and The Board of Trustees of the Leland Stanford
 * Junior University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * o Neither the name of the University of California/California Digital
 *   Library, Ithaka Harbors/Portico, or Stanford University, nor the names of
 *   its contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.jhove2.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * JHOVE2 annotation marking reportable properties. A reportable property is a
 * named, typed value. Properties have two names: a short descriptive name, not
 * necessarily unique; and a formal name guaranteed unique. The descriptive name
 * is implied by the accessor method that returns it. For example, the method
 * <code>String getReportableName()</code> defines a scalar string-valued property named
 * "Name". the formal name is an identifier in JHOVE2 namespace based on the
 * name of the {@link org.jhove2.core.reportable.Reportable} containing the property
 * concatenated with the property's descriptive name.
 * 
 * @author mstrong, slabrams, smorrissey
 */
@Documented
@Inherited
@Retention(RetentionPolicy.RUNTIME)
@Target( { ElementType.METHOD })
public @interface ReportableProperty {
	/** Default description and reference value. */
	public static final String DEFAULT = "Not available.";
	
	/**
	 * Property type: raw or descriptive.  A raw property reports itself
	 * in the exact form that was found in the source unit; a descriptive
	 * property reports itself in a more human-readable form.
	 */
	public enum PropertyType {Default, Raw, Descriptive}

    /**
     * Ordinal position of this property relative to all properties directly
     * defined in a class.
     */
    public int order() default 1;

	/**
	 * Property reference, a citation to an external source document that
	 * defines the property.
	 */
	public String ref() default DEFAULT;
	
	/** Property type: raw or descriptive. */
	public PropertyType type() default PropertyType.Default;
    
    /** Property description. */
    public String value() default DEFAULT;
}
